<?php
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/Pipeline.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/ComponentConfigurations.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/error-handling/HandleError.php" );
class Pipelines {
   private $gDefault;
   
   
   private $gPipelinesList;
   
   private $gComponentConfigurations;
   
   private $gErrorHandler;
      
   
   
   
   public function __construct()
   {
      $this->gPipelinesList = array();
      $this->gComponentConfigurations = NULL;
      $this->gErrorHandler = NULL;
   }
   
   
   
   public function parse( DOMNode $inNode )
   {
      
      $pipelinesDom = new DOMDocument;
      $pipelinesDom->appendChild( $pipelinesDom->importNode( $inNode, 1 ) );
      $xpath = new domxpath( $pipelinesDom );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $nodeList = $xpath->query( "//*" );
      $idx = 0;
      foreach ( $nodeList as $node ) {
         if ( $node->localName == "pipeline" ) {
            $pipeline = new Pipeline();
            $this->gPipelinesList[ $idx++ ] = $pipeline;
            try {
               $pipeline->parse( $node );
            } catch ( UserException $e ) {
               throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
            }
         } else if ( $node->localName == "component-configurations" ) {
            try {
               $this->gComponentConfigurations = new ComponentConfigurations();
               $this->gComponentConfigurations->parse( $node );
            } catch ( UserException $e ) {
               throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
            }
         } else if ( $node->localName == "handle-errors" ) {
            try {
               $this->gErrorHandler = new HandleError();
               $this->gErrorHandler->parse( $node );
            } catch ( UserException $e ) {
               throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
            }
         }
      }
   }
   
   
   
   public function run( $inURL, $inQueryString, $inInternalRequest )
   {
      
      
      foreach( $this->gPipelinesList as $node ) {
         try {
            
            if ( $node->run( $inURL, $inQueryString, $inInternalRequest ) ) return true;
         } catch ( ExitException $e ) {
            throw new ExitException();
         } catch ( UserException $e ) {
            throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
         } catch ( RunException $e ) {
            throw new RunException( $e->getMessage(), $e->getCode(), NULL );
         }
      }
      return false;
   }
   
   
   
   public function getComponentConfigurations()
   {
      return $this->gComponentConfigurations;
   } 
   
   
   
   public function toString()
   {
      $mess = " <pipelines>\n";
      if ( $this->gComponentConfigurations != NULL ) {
         $mess .= $this->gComponentConfigurations->toString();
      }
      foreach ( $this->gPipelinesList as $node ) {
         $mess .= $node->toString();
      }
      if ( $this->gErrorHandler != NULL ) {
         $mess .= $this->gErrorHandler->toString();
      }
      $mess .= " </pipelines>\n";
      return $mess;
   }
}
?>
