<?php
class Resource {
       
       
   private $gName;
    
   private $gPipe;
   
   
   
   public function __construct( $inName, DOMNode $inNode )
   {
       $this->gName = $inName;
       if ( !( strlen( $inName ) > 0  ) ) {
           $dom = new DOMDocument;
           $dom->appendChild( $dom->importNode( $inNode, 1 ) );
           throw new UserException( "Sitemap resource declaration must have name defined", PalooseException::SITEMAP_VIEW_PARSE_ERROR, $dom );
       }
   }
   
   
   
    
   public function parse( DOMNode $inNode )
   {
      
      $resourceDom = new DOMDocument;
      $resourceDom->appendChild( $resourceDom->importNode( $inNode, 1 ) );
      
      
      $xpath = new domxpath( $resourceDom );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      $component = $xpath->query( "//m:resource/*" );
      $this->gPipe = new Pipe();
      foreach ( $component as $pipeNode ) {
         $name = $pipeNode->localName;
         
         switch ( $name ) {
            case "redirect-to" :
            case "mount" :
            case "read" :
               throw new UserException( "Resource pipe cannot have redirect-to, mount or read components", PalooseException::SITEMAP_VIEW_PARSE_ERROR, $resourceDom );
            case "select" :
               try {
                  $selector = PipeElement::createSelector( $pipeNode );
                  $this->gPipe->addComponent( $selector );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "aggregate" :
               try {
                  $aggregator = PipeElement::createAggregator( $pipeNode );
                  
                  $this->gPipe->addComponent( $aggregator );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
               
            case "generate" :
               try {
                  $generator = PipeElement::createGenerator( $pipeNode );
                  
                  $this->gPipe->addComponent( $generator );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "transform" :
               try {
                  $transformer = PipeElement::createTransformer( $pipeNode );
                  
                  $this->gPipe->addComponent( $transformer );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "serialize" :
               try {
                  $serializer = PipeElement::createSerializer( $pipeNode );
                  
                  $this->gPipe->addComponent( $serializer );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
         }
      }
   }
   
   
   
   
    
   public function getName()
   {
      return $this->gName;
   }
   
   
   
    
   public function getPipe()
   {
      return $this->gPipe;
   }
   
   
   
    
   public function toString()
   {
      $mess = "   <resource name='{$this->gName}' >\n";
      $mess .= $this->gPipe->toString();
      $mess .= "  </resource>\n";
      return $mess;
   }
}
?>
