<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class FilterTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
    
   public $gElementName;
   
    
   public $gElementNS;
   
    
   public $gCount;
   
    
   public $gBlockNum;
   
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
       
        
        $this->gElementName = $this->gParameters->getParameter( 'element-name' );
      if ( strlen( $this->gElementName ) == 0 ) {
           throw new UserException( "Must have an element-name attribute present", PalooseException::SITEMAP_TRANSFORMER_PARSE_ERROR, $inDOM );
        }
       
        $this->gCount = $this->gParameters->getParameter( 'count' );
      if ( strlen( $this->gCount ) == 0 ) {
           throw new UserException( "Must have a count attribute present", PalooseException::SITEMAP_TRANSFORMER_PARSE_ERROR, $inDOM );
        }
       
        $this->gBlockNum = $this->gParameters->getParameter( 'blocknr' );
      if ( strlen( $this->gBlockNum ) == 0 ) {
           throw new UserException( "Must have an blocknr attribute present", PalooseException::SITEMAP_TRANSFORMER_PARSE_ERROR, $inDOM );
        }
        
        $colonPos = strrpos( $this->gElementName, ":" );
        if ( $colonPos == 0 ) {
           $this->gElementNS = "";
        } else {
           $this->gElementNS = substr( $this->gElementName, 0, $colonPos );
           $this->gElementName = substr( $this->gElementName, $colonPos + 1 );
        }
    }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, $rootNode, $newElement, Environment::$configuration[ 'sqlNamespace' ] );
         $this->gTranslatedDOM->formatOutput = true;
         
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode, $inSQLNamespace )
   {
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandString( $inVariableStack, $attribute->value );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
      
      
      $children = $inParentNode->childNodes;
      $count = 0;
      $block = 0;
      $lastBlockNode = NULL;
      try {
         foreach ( $children as $child ) {
            if ( $child->nodeType == XML_ELEMENT_NODE ) {
            }
            if ( $child->localName == $this->gElementName && $child->namespaceURI == $this->gElementNS ) {
               if ( $count % $this->gCount == 0 ) {
                  
                  $block++;
                  
                  $blockElement = $this->gTranslatedDOM->createElementNS( $inSQLNamespace, 'block' );
                  $blockNode = $inNewParentNode->appendChild( $blockElement );
                  $blockNode->setAttribute( "id", $block );
                  $lastBlockNode = $blockNode;
               }
               
               if ( $block == $this->gBlockNum ) {
                  if ( $child->nodeType == XML_ELEMENT_NODE ) {
                     
                     $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
                     
                     $this->processNode( $inVariableStack, $child, $newElementNode, $inSQLNamespace );
                     $lastBlockNode->appendChild( $newElementNode );
                  } else if ( $child->nodeType == XML_TEXT_NODE ) {
                     
                     $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
                     $lastBlockNode->appendChild( $newTextNode );
                  }
               }
               $count++;
            } else { 
               
               if ( $child->nodeType == XML_ELEMENT_NODE ) {
                 
                 $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
                 
                 $this->processNode( $inVariableStack, $child, $newElementNode, $inSQLNamespace );
                 $inNewParentNode->appendChild( $newElementNode );
               } else if ( $child->nodeType == XML_TEXT_NODE ) {
                  
                  $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
                  $inNewParentNode->appendChild( $newTextNode );
               }
            }
         }
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
  
    
   
   
  
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _FilterTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
       $this->gGenericType = PipeElement::TRANSFORMER;
        $this->gPackageName = "FilterTransformer";
      
   }
}
?>
