<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
 
class LogTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
       
   private $gAppend;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
      if ( $this->gParameters->getParameter( "logfile" ) == NULL ) {
         throw new UserException( "Must have logfile attribute",
            PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR,
            $inDOM );
      } else {
         $filename = StringResolver::expandPseudoProtocols( $this->gParameters->getParameter( "logfile" ) );
         $this->gParameters->setParameter( "logfile", $filename );
      }
      if ( $this->gParameters->getParameter( "append" ) == NULL ) {
         $this->gParameters->setParameter( "append", "yes" );
      }
      $this->gAppend = Utilities::normalizeBoolean( $this->gParameters->getParameter( "append" ) );
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $fileName = $this->gParameters->getParameter( "logfile" );
      if ( $this->gAppend == 0 ) {
         if ( !$fileHandle = fopen( $fileName, "w" ) ) {
              throw new RunException( "Cannot open log file for overwrite: $fileName", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
         }
      } else {
         if ( !$fileHandle = fopen( $fileName, "a" ) ) {
              throw new RunException( "Cannot open log file for append: $fileName", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
         }
      }
      $filterXPath = $this->gParameters->getParameter( "filter" );
      $xpath = new domxpath( $inDOM );
      if ( $filterXPath == NULL ) {
          $filterXPath = "/*";  
      }
      $logNode = Utilities::getXPathListNode( 0, $xpath, $filterXPath );
      $logString = $inDOM->saveXML( $logNode );
      
      
      if ( fwrite( $fileHandle, $logString ) === false ) {
           throw new RunException( "Cannot write to log file: $fileName", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      if ( !fclose( $fileHandle ) ) {
           throw new RunException( "Cannot close log file: $fileName", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
      }
      return $inDOM;
   }
   
   
   
  
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _LogTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "LogTransformer";
   }
}
?>
